@@ -181,8 +181,9 @@ def generate_parse(print_function):
             return statement
 
     @pg.production("small_stmt : expr_stmt")
-    @pg.production("expr_stmt : testlist")
+    @pg.production("expr_stmt : testlist_star_expr")
     @pg.production("testlist : test")
+    @pg.production("testlist_star_expr : test_or_star_expr")
     @pg.production("test : or_test")
     @pg.production("test : lambdef")
     @pg.production("or_test : and_test")
@@ -285,6 +286,7 @@ def generate_parse(print_function):
         }]
 
     @pg.production("classdef : CLASS NAME LEFT_PARENTHESIS testlist RIGHT_PARENTHESIS COLON suite")
+    @pg.production("classdef : CLASS NAME LEFT_PARENTHESIS argslist RIGHT_PARENTHESIS COLON suite")
     def class_stmt_inherit(pack,):
         (class_, name, left_parenthesis, testlist, right_parenthesis, colon, suite) = pack
         return [{
@@ -380,9 +382,9 @@ def generate_parse(print_function):
             "formatting": at.hidden_tokens_after,
         }] + endl
 
-    @pg.production("funcdef : async_maybe DEF NAME LEFT_PARENTHESIS typed_parameters RIGHT_PARENTHESIS COLON suite")
+    @pg.production("funcdef : async_maybe DEF NAME LEFT_PARENTHESIS typed_parameters RIGHT_PARENTHESIS return_annotation COLON suite")
     def function_definition(pack):
-        (async_maybe, def_, name, left_parenthesis, typed_parameters, right_parenthesis, colon, suite) = pack
+        (async_maybe, def_, name, left_parenthesis, typed_parameters, right_parenthesis, return_annotation, colon, suite) = pack
 
         if async_maybe["async"] and async_maybe["value"] != "async":
             raise ParsingError("The only possible keyword before a 'def' is 'async', not '%s'" % async_maybe["value"])
@@ -390,6 +392,9 @@ def generate_parse(print_function):
         return [{
             "type": "def",
             "async": async_maybe["async"],
+            "return_annotation": return_annotation["value"],
+            "return_annotation_first_formatting": return_annotation["first_formatting"],
+            "return_annotation_second_formatting": return_annotation["second_formatting"],
             "async_formatting": async_maybe.get("formatting", []),
             "decorators": [],
             "name": name.value,
@@ -403,6 +408,23 @@ def generate_parse(print_function):
             "value": suite,
         }]
 
+    @pg.production("return_annotation : ")
+    def return_annotation_empty(pack):
+        return {
+            "value": {},
+            "first_formatting": [],
+            "second_formatting": [],
+        }
+
+    @pg.production("return_annotation : RIGHT_ARROW test")
+    def return_annotation(pack):
+        right_arrow, test = pack
+        return {
+            "value": test,
+            "first_formatting": right_arrow.hidden_tokens_before,
+            "second_formatting": right_arrow.hidden_tokens_after,
+        }
+
     @pg.production("argslist : argslist argument")
     @pg.production("typed_parameters : typed_parameters typed_parameter")
     @pg.production("parameters : parameters parameter")
@@ -431,31 +453,15 @@ def generate_parse(print_function):
             "value": name_.value,
         }
 
-    @pg.production("typed_name : NAME COLON test")
-    def typed_name_with_type(pack):
-        (name_, colon, test) = pack
-        return {
-            "type": "typed_name",
-            "first_formatting": colon.hidden_tokens_before if colon else [],
-            "second_formatting": colon.hidden_tokens_after if colon else [],
-            "value": name_.value,
-            "annotation": test
-        }
-
-    @pg.production("typed_name : NAME")
-    def typed_name_no_type(pack):
-        (name_,) = pack
-        return {
-            "type": "name",
-            "value": name_.value,
-        }
-
     @pg.production("typed_parameter : LEFT_PARENTHESIS name RIGHT_PARENTHESIS maybe_test")
     @pg.production("parameter : LEFT_PARENTHESIS name RIGHT_PARENTHESIS maybe_test")
     def parameter_fpdef(pack):
         (left_parenthesis, name, right_parenthesis, (equal, test)) = pack
         return [{
             "type": "def_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "first_formatting": equal.hidden_tokens_before if equal else [],
             "second_formatting": equal.hidden_tokens_after if equal else [],
             "value": test,
@@ -475,6 +481,9 @@ def generate_parse(print_function):
         (left_parenthesis, fplist, right_parenthesis, (equal, test)) = pack
         return [{
             "type": "def_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "first_formatting": equal.hidden_tokens_before if equal else [],
             "second_formatting": equal.hidden_tokens_after if equal else [],
             "value": test,
@@ -518,12 +527,43 @@ def generate_parse(print_function):
             "target": name if equal else {}
         }]
 
-    @pg.production("typed_parameter : typed_name maybe_test")
-    @pg.production("parameter : name maybe_test")
-    def parameter_with_default(pack):
-        (name, (equal, test)) = pack
+    @pg.production("typed_parameter : name COLON test maybe_test")
+    def parameter_annotation_with_default(pack):
+        # name, (equal, test) = pack
+        name, colon, annotation, (equal, test) = pack
         return [{
             "type": "def_argument",
+            "annotation": annotation,
+            "annotation_first_formatting": colon.hidden_tokens_before,
+            "annotation_second_formatting": colon.hidden_tokens_after,
+            "first_formatting": equal.hidden_tokens_before if equal else [],
+            "second_formatting": equal.hidden_tokens_after if equal else [],
+            "value": test,
+            "target": name
+        }]
+
+    @pg.production("typed_parameter : name maybe_test")
+    def parameter_alone_with_default(pack):
+        name, (equal, test) = pack
+        return [{
+            "type": "def_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
+            "first_formatting": equal.hidden_tokens_before if equal else [],
+            "second_formatting": equal.hidden_tokens_after if equal else [],
+            "value": test,
+            "target": name
+        }]
+
+    @pg.production("parameter : name maybe_test")
+    def parameter_with_default(pack):
+        name, (equal, test) = pack
+        return [{
+            "type": "def_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "first_formatting": equal.hidden_tokens_before if equal else [],
             "second_formatting": equal.hidden_tokens_after if equal else [],
             "value": test,
@@ -552,6 +592,9 @@ def generate_parse(print_function):
         (star, test,) = pack
         return [{
             "type": "list_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "formatting": star.hidden_tokens_after,
             "value": test,
         }]
@@ -561,6 +604,9 @@ def generate_parse(print_function):
         (double_star, test,) = pack
         return [{
             "type": "dict_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "formatting": double_star.hidden_tokens_after,
             "value": test,
         }]
@@ -571,12 +617,12 @@ def generate_parse(print_function):
         return [{
             "type": "list_argument",
             "formatting": star.hidden_tokens_after,
+            "annotation": test,
+            "annotation_first_formatting": colon.hidden_tokens_before if colon else [],
+            "annotation_second_formatting": colon.hidden_tokens_after if colon else [],
             "value": {
-                "type": "typed_name",
-                "first_formatting": colon.hidden_tokens_before if colon else [],
-                "second_formatting": colon.hidden_tokens_after if colon else [],
+                "type": "name",
                 "value": name.value,
-                "annotation": test
             }
         }]
 
@@ -586,12 +632,12 @@ def generate_parse(print_function):
         return [{
             "type": "dict_argument",
             "formatting": double_star.hidden_tokens_after,
+            "annotation": test,
+            "annotation_first_formatting": colon.hidden_tokens_before if colon else [],
+            "annotation_second_formatting": colon.hidden_tokens_after if colon else [],
             "value": {
-                "type": "typed_name",
-                "first_formatting": colon.hidden_tokens_before if colon else [],
-                "second_formatting": colon.hidden_tokens_after if colon else [],
+                "type": "name",
                 "value": name.value,
-                "annotation": test
             }
         }]
 
@@ -602,6 +648,9 @@ def generate_parse(print_function):
         (star, name,) = pack
         return [{
             "type": "list_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "formatting": star.hidden_tokens_after,
             "value": {
                 "type": "name",
@@ -625,6 +674,9 @@ def generate_parse(print_function):
         (double_star, name,) = pack
         return [{
             "type": "dict_argument",
+            "annotation": {},
+            "annotation_first_formatting": [],
+            "annotation_second_formatting": [],
             "formatting": double_star.hidden_tokens_after,
             "value": {
                 "type": "name",
@@ -763,7 +815,7 @@ def generate_parse(print_function):
         if print_function:
             new_tokens = []
             for token in tokens:
-                if token[0] == "PRINT":
+                if token[0] in ("PRINT", "EXEC"):
                     token = list(token)
                     token[0] = "NAME"
                     token = tuple(token)
