@@ -14,19 +14,17 @@ def include_primivites(pg, print_function):
                 "formatting": [],
             }
 
-
         @pg.production("print_stmt : PRINT testlist")
         def print_stmt(pack):
             (print_, testlist) = pack
             return {
                 "type": "print",
-                "value": testlist["value"] if testlist["type"] == "tuple" and testlist["with_parenthesis"] == False else [testlist],
+                "value": testlist["value"] if testlist["type"] == "tuple" and testlist["with_parenthesis"] is False else [testlist],
                 "destination": None,
                 "destination_formatting": [],
                 "formatting": print_.hidden_tokens_after,
             }
 
-
         @pg.production("print_stmt : PRINT RIGHT_SHIFT test")
         def print_stmt_redirect(pack):
             (print_, right_shift, test) = pack
@@ -38,7 +36,6 @@ def include_primivites(pg, print_function):
                 "formatting": print_.hidden_tokens_after,
             }
 
-
         @pg.production("print_stmt : PRINT RIGHT_SHIFT test COMMA testlist")
         def print_stmt_redirect_testlist(pack):
             (print_, right_shift, test, comma, testlist) = pack
@@ -47,7 +44,6 @@ def include_primivites(pg, print_function):
                 "first_formatting": comma.hidden_tokens_before,
                 "second_formatting": comma.hidden_tokens_after,
             }]
-            #print testlist
             value += testlist["value"] if testlist["type"] == "tuple" else [testlist]
             return {
                 "type": "print",
@@ -57,6 +53,50 @@ def include_primivites(pg, print_function):
                 "formatting": print_.hidden_tokens_after,
             }
 
+        @pg.production("assert_stmt : EXEC expr")
+        def exec_stmt(pack):
+            (exec_, expr) = pack
+            return {
+                "type": "exec",
+                "value": expr,
+                "globals": None,
+                "locals": None,
+                "first_formatting": exec_.hidden_tokens_after,
+                "second_formatting": [],
+                "third_formatting": [],
+                "fourth_formatting": [],
+                "fifth_formatting": []
+            }
+
+        @pg.production("assert_stmt : EXEC expr IN test")
+        def exec_stmt_in(pack):
+            (exec_, expr, in_, test) = pack
+            return {
+                "type": "exec",
+                "value": expr,
+                "globals": test,
+                "locals": None,
+                "first_formatting": exec_.hidden_tokens_after,
+                "second_formatting": in_.hidden_tokens_before,
+                "third_formatting": in_.hidden_tokens_after,
+                "fourth_formatting": [],
+                "fifth_formatting": []
+            }
+
+        @pg.production("assert_stmt : EXEC expr IN test COMMA test")
+        def exec_stmt_in_comma(pack):
+            (exec_, expr, in_, test, comma, test2) = pack
+            return {
+                "type": "exec",
+                "value": expr,
+                "globals": test,
+                "locals": test2,
+                "first_formatting": exec_.hidden_tokens_after,
+                "second_formatting": in_.hidden_tokens_before,
+                "third_formatting": in_.hidden_tokens_after,
+                "fourth_formatting": comma.hidden_tokens_before,
+                "fifth_formatting": comma.hidden_tokens_after
+            }
 
     @pg.production("flow_stmt : return_stmt")
     @pg.production("flow_stmt : break_stmt")
@@ -67,7 +107,6 @@ def include_primivites(pg, print_function):
         (flow_stmt,) = pack
         return flow_stmt
 
-
     @pg.production("return_stmt : RETURN")
     def return_empty(pack):
         (token,) = pack
@@ -77,7 +116,6 @@ def include_primivites(pg, print_function):
             "formatting": token.hidden_tokens_after,
         }
 
-
     @pg.production("yield_expr : YIELD")
     def yield_expr(pack):
         (yield_,) = pack
@@ -87,7 +125,6 @@ def include_primivites(pg, print_function):
             "formatting": yield_.hidden_tokens_after,
         }
 
-
     @pg.production("break_stmt : BREAK")
     @pg.production("continue_stmt : CONTINUE")
     @pg.production("pass_stmt : PASS")
@@ -95,7 +132,6 @@ def include_primivites(pg, print_function):
         (token,) = pack
         return {"type": token.name.lower()}
 
-
     @pg.production("raise_stmt : RAISE")
     def raise_stmt_empty(pack):
         (raise_,) = pack
@@ -108,10 +144,10 @@ def include_primivites(pg, print_function):
             "second_formatting": [],
             "third_formatting": [],
             "fourth_formatting": [],
-            "fifth_formatting": []
+            "fifth_formatting": [],
+            "comma_or_from": None,
         }
 
-
     @pg.production("raise_stmt : RAISE test")
     def raise_stmt(pack):
         (raise_, test) = pack
@@ -124,9 +160,25 @@ def include_primivites(pg, print_function):
             "second_formatting": [],
             "third_formatting": [],
             "fourth_formatting": [],
-            "fifth_formatting": []
+            "fifth_formatting": [],
+            "comma_or_from": None,
         }
 
+    @pg.production("raise_stmt : RAISE test FROM test")
+    def raise_stmt_from(pack):
+        (raise_, test, from_, test2) = pack
+        return {
+            "type": "raise",
+            "value": test,
+            "instance": test2,
+            "traceback": None,
+            "first_formatting": raise_.hidden_tokens_after,
+            "second_formatting": from_.hidden_tokens_before,
+            "third_formatting": from_.hidden_tokens_after,
+            "fourth_formatting": [],
+            "fifth_formatting": [],
+            "comma_or_from": "from",
+        }
 
     @pg.production("raise_stmt : RAISE test COMMA test")
     def raise_stmt_instance(pack):
@@ -140,10 +192,10 @@ def include_primivites(pg, print_function):
             "second_formatting": comma.hidden_tokens_before,
             "third_formatting": comma.hidden_tokens_after,
             "fourth_formatting": [],
-            "fifth_formatting": []
+            "fifth_formatting": [],
+            "comma_or_from": ",",
         }
 
-
     @pg.production("raise_stmt : RAISE test COMMA test COMMA test")
     def raise_stmt_instance_traceback(pack):
         (raise_, test, comma, test2, comma2, test3) = pack
@@ -156,58 +208,10 @@ def include_primivites(pg, print_function):
             "second_formatting": comma.hidden_tokens_before,
             "third_formatting": comma.hidden_tokens_after,
             "fourth_formatting": comma2.hidden_tokens_before,
-            "fifth_formatting": comma2.hidden_tokens_after
+            "fifth_formatting": comma2.hidden_tokens_after,
+            "comma_or_from": ",",
         }
 
-
-    @pg.production("assert_stmt : EXEC expr")
-    def exec_stmt(pack):
-        (exec_, expr) = pack
-        return {
-            "type": "exec",
-            "value": expr,
-            "globals": None,
-            "locals": None,
-            "first_formatting": exec_.hidden_tokens_after,
-            "second_formatting": [],
-            "third_formatting": [],
-            "fourth_formatting": [],
-            "fifth_formatting": []
-        }
-
-
-    @pg.production("assert_stmt : EXEC expr IN test")
-    def exec_stmt_in(pack):
-        (exec_, expr, in_, test) = pack
-        return {
-            "type": "exec",
-            "value": expr,
-            "globals": test,
-            "locals": None,
-            "first_formatting": exec_.hidden_tokens_after,
-            "second_formatting": in_.hidden_tokens_before,
-            "third_formatting": in_.hidden_tokens_after,
-            "fourth_formatting": [],
-            "fifth_formatting": []
-        }
-
-
-    @pg.production("assert_stmt : EXEC expr IN test COMMA test")
-    def exec_stmt_in_comma(pack):
-        (exec_, expr, in_, test, comma, test2) = pack
-        return {
-            "type": "exec",
-            "value": expr,
-            "globals": test,
-            "locals": test2,
-            "first_formatting": exec_.hidden_tokens_after,
-            "second_formatting": in_.hidden_tokens_before,
-            "third_formatting": in_.hidden_tokens_after,
-            "fourth_formatting": comma.hidden_tokens_before,
-            "fifth_formatting": comma.hidden_tokens_after
-        }
-
-
     @pg.production("assert_stmt : ASSERT test")
     def assert_stmt(pack):
         (assert_, test) = pack
@@ -220,7 +224,6 @@ def include_primivites(pg, print_function):
             "third_formatting": []
         }
 
-
     @pg.production("assert_stmt : ASSERT test COMMA test")
     def assert_stmt_message(pack):
         (assert_, test, comma, test2) = pack
@@ -233,7 +236,6 @@ def include_primivites(pg, print_function):
             "third_formatting": comma.hidden_tokens_after
         }
 
-
     @pg.production("global_stmt : GLOBAL names")
     def global_stmt(pack):
         (global_, names) = pack
@@ -243,7 +245,6 @@ def include_primivites(pg, print_function):
             "value": names,
         }
 
-
     @pg.production("nonlocal_stmt : NONLOCAL names")
     def nonlocal_stmt(pack):
         (token, names) = pack
@@ -253,25 +254,16 @@ def include_primivites(pg, print_function):
             "value": names,
         }
 
-
     @pg.production("names : NAME")
     def names_name(pack):
         (name,) = pack
         return [create_node_from_token(name)]
 
-
-    @pg.production("names : NAME")
-    def names_name(pack):
-        (name,) = pack
-        return [create_node_from_token(name)]
-
-
     @pg.production("names : names comma name")
     def names_names_name(pack):
         (names, comma, name,) = pack
         return names + [comma, name]
 
-
     @pg.production("return_stmt : RETURN testlist")
     @pg.production("yield_expr : YIELD testlist")
     @pg.production("del_stmt : DEL exprlist")
@@ -283,7 +275,6 @@ def include_primivites(pg, print_function):
             "formatting": token.hidden_tokens_after,
         }
 
-
     @pg.production("yield_expr : YIELD FROM test")
     def yield_from_expr(pack):
         (yield_, from_, test) = pack
